module EightPuzzle exposing (informed, start)

import Search
import Random.List
import Random
import List.Extra exposing (swapAt, elemIndex)
import Html exposing (text)


main =
    text <|
        toString <|
            Search.nextN 250 <|
                Search.iterativeDeepening 1 informed [ start 3 3 ]



{- The puzzles state -}


type alias State =
    { board : List Int
    , xSize : Int
    , ySize : Int
    }



{- Constructors for goal and random board states. -}


seed =
    Random.initialSeed 10


offset x y width =
    y * width + x


goalList : Int -> Int -> List Int
goalList x y =
    List.range 0 (x * y)


shuffled : Int -> Int -> List Int
shuffled x y =
    goalList x y
        |> Random.List.shuffle
        |> (flip Random.step) seed
        |> Tuple.first


start : Int -> Int -> State
start x y =
    let
        swaps =
            countSwaps <| shuffled x y
    in
        { board = shuffled x y
        , xSize = x
        , ySize = y
        }


swap : Int -> Int -> List a -> List a
swap a b list =
    swapAt a b list
        |> Maybe.withDefault list


countSwaps : List Int -> Int
countSwaps list =
    let
        d =
            Debug.log "countSwaps" <|
                List.map (\tile -> elemIndex tile list) (goalList 3 3)
    in
        0



{- Board operations and goal checks. -}


type Move
    = Up
    | Down
    | Left
    | Right


move : Move -> State -> Maybe ( State, Bool )
move move state =
    Nothing


goal : State -> Bool
goal state =
    state.board == goalList state.xSize state.ySize


step : Search.Step State
step state =
    [ ( state, True ) ]



{- Packages the search operators as an informed search. -}


informed : Search.Informed State
informed =
    { step = step
    , cost = \_ -> 1.0
    , heuristic = \_ -> 1.0
    }
